      PROGRAM MPNE1D
C     ******************************************************************
C     ****                                                          ****
C     ****                      M P N E 1 D                         ****
C     ****                      ===========                         ****
C     ****                ANALYTICAL SOLUTION FOR                   ****
C     ****        MULTI-PROCESS NON-EQUILIBRIUM SORPTION            ****
C     ****                                                          ****
C     ******************************************************************
C
c     DESCRIPTION
c     ===========
C     ANALYTICAL SOLUTION FOR 1-D ADVECTIVE-DISPERSIVE TRANSPORT
C     WITH MULTI-PROCESS NON-EQUILIBRIUM SORPTION.
C      (1) DIMENSIONAL SOLUTION
C      (2) FIRST-ORDER DECAY REACTIONS
C      (3) NON-ZERO INITIAL CONCENTRATIONS
C      (4) TYPE I or TYPE III INFLOW BOUNDARY CONDITION WITH
C          TIME-VARYING INFLOW CONCENTRATION
C      (5) SEMI-INFINITE OR FINITE COLUMN, WITH TYPE I OR TYPE II
C          OUTFLOW BOUNDARY CONDITION
C
C     REF:  BRUSSEAU, M.L., R.E. JESSUP, AND P.S.C. RAO,
C           MODELING SOLUTE TRANSPORT INFLUENCED BY MULTIPROCESS
C           NONEQUILIBRIUM AND TRANSFORMATION REACTIONS.
C           WATER RESOURCES RESEARCH, 28(1), 175-182, 1992.
C
C     DEVELOPMENT HISTORY
C     ===================
c     VERSION 1    : C.J. NEVILLE 1992/03
C     VERSION 2    : CJN 1992/09
C     UPDATED      : CJN 1992/09/29
C                    CJN 1993/03/16
C                    CJN 1993/06/10
c                    CJN 1998/11/19
C     VERSION 3    : CJN 2000/08
C     VERSION 3.1  : CJN 2002/01,
C                    with modifications for Fortran90 by J.P. KEIZER
C     VERSION 3.2  : CJN 2003/01
C     VERSION 4.1  : CJN 2004/08/24
C
C     DEFINITION OF INPUT PARAMETERS
c     ==============================
C     Rhob   : BULK DENSITY  (M/L**3)
C     THETA  : TOTAL WATER CONTENT (L**3/L**3)
C     Q      : DARCY FLUX    (L/T)
C     D      : HYDRODYNAMIC DISPERSION COEFFICIENT (L**/T)
C     PHI    : PROPORTION OF PORE WATER THAT IS MOBILE
C     F      : MASS FRACTION OF SORBENT COMPRISING MOBILE REGION (-)
C     ALfa   : FIRST-ORDER MASS TRANSFER COEFFICIENT (1/T)
C     FM     : FRACTION OF SORBENT IN MOBILE REGION FOR WHICH SORPTION
C              IS INSTANTANEOUS
C     FIM    : FRACTION OF SORBENT IN IMMOBILE REGION FOR WHICH SORPTION IS
C              INSTANTANEOUS
C     KM     : EQUILIBRIUM SORPTION CONSTANT IN MOBILE REGION (L**3/M)
C     KIM    : EQUILIBRIUM SORPTION CONSTANT IN IMMOBILE REGION (L**3/M)
C     km2    : FIRST-ORDER SORPTION KINETIC COEFF. FOR MOBILE REGION (1/T)
C     kim2   : FIRST-ORDER SORPTION KINETIC COEFF. FOR IMMOBILE REGION (1/T)
C     LM     : FIRST-ORDER DECAY CONSTANT FOR MOBILE REGION (1/T)
C     LM1    : FIRST-ORDER DECAY CONSTANT FOR INSTANTANEOUS SORPTION
C              SITES IN MOBILE REGION (1/T)
C     LM2    : FIRST-ORDER DECAY CONSTANT FOR RATE-LIMITED SORPTION
C              SITES IN MOBILE REGION (1/T)
C     LIM    : FIRST-ORDER DECAY CONSTANT FOR IMMOBILE REGION (1/T)
C     LIM1   : FIRST-ORDER DECAY CONSTANT FOR INSTANTANEOUS SORPTION
C              SITES IN IMMOBILE REGION (1/T)
C     LIM2   : FIRST-ORDER DECAY CONSTANT FOR RATE-LIMITED SORPTION
C              SITES IN IMMOBILE REGION (1/T)
c     IBC    : INFLOW BOUNDARY CONDITION
C              = 1 : TYPE I   (DIRICHLET)
C              = 3 : TYPE III (CAUCHY)
C     NP     : NUMBER OF POINTS DESCRIBING INFLOW CONCENTRATION HISTORY
C     TI,CI  : INFLOW CONCENTRATION HISTORY POINT
C     OBC    : OUTFLOW BOUNDARY CONDITION
C              = 1 : SEMI-INFINITE DOMAIN
C              = 2 : FINITE DOMAIN, TYPE II
c              = 3 : FINITE DOMAIN, TYPE I
C     LENGTH : LENGTH OF THE DOMAIN (USED ONLY FOR FINITE DOMAIN)
C     CL     : CONCENTRATION AT OUTFLOW BOUNDARY FOR FINITE DOMAIN-TYPE I
C     CM0    : INITIAL CONCENTRATION IN MOBILE REGION DISSOLVED PHASE
C     CIM0   : INITIAL CONCENTRATION IN IMMOBILE REGION DISSOLVED PHASE
C     SM20   : INITIAL CONCENTRATION IN MOBILE REGION RATE-LIMITED
C              SORBED PHASE
C     SIM20  : INITIAL CONCENTRATION IN THE MOBILE REGION RATE-LIMITED
C              SORBED PHASE
C     XX     : DISTANCE FROM COLUMN INLET (L)
C     XMIN   : MINIMUM DISTANCE OF INTEREST (L)
C     XMAX   : MAXIMUM DISTANCE OF INTEREST (L)
C     DX     : DISTANCE INCREMENT (L)
C     T      : TIME SINCE START OF TRANSPORT (T)
C     TMIN   : MINIMUM TIME OF INTEREST (T)
C     TMAX   : MAXIMUM TIME OF INTEREST (T)
C     DT     : TIME INCREMENT (T)
C
C     DECLARATION OF VARIABLES
C     ========================
      IMPLICIT NONE
      INTEGER MAXPT,MAXP,MAXB
      PARAMETER(MAXPT=50,MAXP=100,MAXB=100)
      DOUBLE PRECISION PARAM(19)
      DOUBLE PRECISION CM0,CIM0,SM20,SIM20,DELTA,LENGTH
      DOUBLE PRECISION CL
      DOUBLE PRECISION TI(MAXPT),CI(MAXPT),TS(MAXPT),DELC(MAXPT)
      DOUBLE PRECISION RHOB,THETA,Q,D,PHI,F,ALFA,FM,FIM,
     1                 KM,KIM,KM2,KIM2,LM,LM1,LM2,LIM,LIM1,LIM2
      DOUBLE PRECISION R,P,OMEGA,BETA(4),KM0,KIM0,EM1,EM2,EIM1,EIM2
      DOUBLE PRECISION XMIN,XMAX,DX,XX,XB(MAXB)
      DOUBLE PRECISION TMIN,TMAX,DT,TIME,TP(MAXP)
      DOUBLE PRECISION CM,CIM
      INTEGER IBC,OBC,NP,I,J,N,LENFIL
      INTEGER NPRO,NXP,NBTC,NTB
      INTEGER IC
      CHARACTER*70 JOBID,TITLE
c
      COMMON /ICS/CM0,CIM0,SM20,SIM20
      COMMON /BCS/TIME,DELTA,TS,DELC,NP,OBC,LENGTH,CL,XX
      COMMON /MPNE/PARAM
      COMMON /CSWITCH/IC
C
C     OPEN INPUT, OUTPUT AND PLOT FILES
C     =================================
      WRITE(*,*) 'input JOBID for file definitions: '
      READ (*,100) JOBID
      WRITE(*,*)
C
      LENFIL=INDEX(JOBID,' ')-1
C
      OPEN(UNIT=55,FILE=JOBID(:LENFIL)//'.inp',STATUS='OLD')
      OPEN(UNIT=66,FILE=JOBID(:LENFIL)//'.out',STATUS='UNKNOWN')
      OPEN(UNIT=67,FILE=JOBID(:LENFIL)//'.dat',STATUS='UNKNOWN')
C
C     READ TITLE OF SIMULATION
C     ========================
      READ(55,100) TITLE
C
C     READ INPUT PARAMETERS
C     =====================
C     BASIC PROBLEM DATA
C     ------------------
      READ(55,*) Rhob
      READ(55,*) THETA
      READ(55,*) Q
      READ(55,*) D
C
C     MPNE DATA
C     ---------
      READ(55,*) PHI
      READ(55,*) F
      READ(55,*) ALFA
      READ(55,*) FM
      READ(55,*) FIM
      READ(55,*) KM
      READ(55,*) KIM
      READ(55,*) KM2
      READ(55,*) KIM2
C
C     FIRST-ORDER DECAY COEFFICIENTS
C     ------------------------------
      READ(55,*) LM
      READ(55,*) LM1
      READ(55,*) LM2
      READ(55,*) LIM
      READ(55,*) LIM1
      READ(55,*) LIM2
C
C     INFLOW BOUNDARY CONDITION DATA
C     ------------------------------
      READ(55,*) IBC
      READ(55,*) NP
      READ(55,*) (TI(N),CI(N), N=1,NP)
c
C     OUTFLOW BOUNDARY CONDITION DATA
C     -------------------------------
      READ(55,*) OBC
      READ(55,*) LENGTH    ! used only for OBC=2 or 3
      READ(55,*) CL        ! used only for OBC=3
C
C     INITIAL CONCENTRATIONS
C     ----------------------
      READ(55,*) CM0
      READ(55,*) CIM0
      READ(55,*) SM20
      READ(55,*) SIM20
C
C     OUTPUT SPECIFICATIONS
C     ---------------------
      READ(55,*) NPRO
      IF(NPRO.GT.0) THEN
         READ(55,*) xmin,xmax,dx
         READ(55,*) (TP(I),I=1,NPRO)
      END IF
C
      READ(55,*) NBTC
      IF(NBTC.GT.0) THEN
         READ(55,*) tmin,tmax,dt
         READ(55,*) (XB(I),I=1,NBTC)
      END IF
C
C     ECHO INPUT PARAMETERS
C     =====================
      WRITE(66,210)
      WRITE(66,211) TITLE
      WRITE(66,220) Rhob,THETA,Q,D
      WRITE(66,221) PHI,F,ALFA,FM,FIM,KM,KIM,KM2,KIM2
      WRITE(66,222) LM,LM1,LM2,LIM,LIM1,LIM2
      WRITE(66,231) CM0,CIM0,SM20,SIM20
C
C     SET INFLOW BOUNDARY CONDITION
C     =============================
      IF(IBC.EQ.3) THEN
         DELTA = 1.D0                  ! TYPE III INFLOW B.C.
      ELSE
         DELTA = 0.D0                  ! TYPE I INFLOW B.C.
      END IF
C
C     "LOAD" MPNE PARAMETER ARRAY
C     ===========================
      PARAM( 1) = RHOB
      PARAM( 2) = THETA
      PARAM( 3) = Q
      PARAM( 4) = D
      PARAM( 5) = PHI
      PARAM( 6) = F
      PARAM( 7) = ALFA
      PARAM( 8) = FM
      PARAM( 9) = FIM
      PARAM(10) = KM
      PARAM(11) = KIM
      PARAM(12) = KM2
      PARAM(13) = KIM2
      PARAM(14) = LM
      PARAM(15) = LM1
      PARAM(16) = LM2
      PARAM(17) = LIM
      PARAM(18) = LIM1
      PARAM(19) = LIM2
C
C     CALCULATE DIMENSIONLESS PARAMETERS
C     ==================================
      R = 1.D0+(RHOB/THETA)*(f*KM+(1.D0-f)*KIM)
      P = Q*length/(PHI*THETA*D)
C
      IF(DABS(Q).GT.0.D0) THEN
         OMEGA = alfa*length/Q
      ELSE
         OMEGA = 1.D6
      END IF
C
      BETA(1) = (PHI+(F*Rhob/THETA)*FM*KM)/R
      BETA(2) = ((F*Rhob/THETA)*(1.D0-FM)*KM)/R
      BETA(3) = (1.D0-PHI+((1.D0-F)*Rhob/THETA)*FIM*KIM)/R
      BETA(4) = (((1.D0-F)*Rhob/THETA)*(1.D0-FIM)*KIM)/R
C
      IF(DABS(Q).GT.0.D0) THEN
         KM0  = (KM2*length*THETA/Q)*(R*BETA(2))
         KIM0 = (KIM2*length*THETA/Q)*(R*BETA(4))
C
         EM1  = (length*THETA/Q)*((PHI*LM)+(F*RHOB*LM1*FM*KM/THETA))
         EM2  = (length*THETA/Q)*(F*RHOB*(1.D0-FM)*KM*LM2/THETA)
         EIM1 = (length*THETA/Q)*(((1.D0-PHI)*LIM)
     1          +((1.D0-F)*RHOB*LIM1*FIM*KIM/THETA))
         EIM2 = (length*THETA/Q)*((1.D0-F)*RHOB
     1          *(1.D0-FIM)*KIM*LIM2/THETA)
      ELSE
         KM0  = 1.D6
         KIM0 = 1.D6
C
         EM1  = 1.D6
         EM2  = 1.D6
         EIM1 = 1.D6
         EIM2 = 1.D6
      END IF
C
C     PRINT OUT COMPUTED DIMENSIONLESS PARAMETERS
C     -------------------------------------------
      WRITE(66,240) R,P,OMEGA,KM0,KIM0
      WRITE(66,241) (BETA(I),I=1,4)
      WRITE(66,242) EM1,EM2,EIM1,EIM2
C
c     CALCULATE INFLOW CONCENTRATION HISTORY
c     ======================================
      ts(1)   = ti(1)
      delc(1) = ci(1)
      if(np.gt.1) then
        do n=2,np
           ts(n)   = (ti(n)+ti(n-1))/2.d0
           delc(n) = ci(n)-ci(n-1)
        end do
      end if
c
c     ECHO CONCENTRATION HISTORY
c     --------------------------
      WRITE(66,230) IBC
      write(66,1234)
      do n=1,np
         if (n.lt.np) then
            write(66,1235) ts(n),ts(n+1),ci(n)
         else
            write(66,1236) ts(n),ci(n)
         end if
      end do
C
C     ECHO OUTFLOW BOUNDARY CONDITION DATA
C     ====================================
      WRITE(66,2231) OBC,LENGTH
      WRITE(66,2232) CL
C
C     WRITE OUTPUT HEADING
C     ====================
      WRITE(66,500)
C
C     CALCULATE CONCENTRATION PROFILES
C     ================================
      IF(NPRO.GT.0) THEN
         NXP = idint(((XMAX-XMIN)/DX)+0.5d0)
         DO I=1,NPRO
C
            TIME = TP(I)
C
C           DISTANCE LOOP
C           -------------
            DO J=0,NXP
               XX = XMIN+DBLE(J)*DX
C
               IF(TIME.LE.0.D0) THEN
                  CM  = CM0
                  CIM = CIM0
               ELSE
                  IC = 1
                  CALL HOOGD(TIME,CM)
                  IC = 2
                  CALL HOOGD(TIME,CIM)
               END IF
C
               WRITE (66,510) TIME,XX,CM,CIM
               WRITE (67,510) TIME,XX,CM,CIM
            END DO
         END DO
      END IF
C
C     CALCULATE BREAKTHROUGH CURVES
C     =============================
      IF(NBTC.GT.0) THEN
         NTB = idint(((TMAX-TMIN)/DT)+0.5d0)
         DO I=1,NBTC
C
            XX = XB(I)
C
C           TIME LOOP
C           ---------
            DO J=0,NTB
               TIME = TMIN+DBLE(J)*DT
C
               IF(TIME.LE.0D0) THEN
                  CM  = CM0
                  CIM = CIM0
               ELSE
                  IC = 1
                  CALL HOOGD(TIME,CM)
                  IC = 2
                  CALL HOOGD(TIME,CIM)
               END IF
C
               WRITE (66,510) TIME,XX,CM,CIM
               WRITE (67,510) TIME,XX,CM,CIM
            END DO
         END DO
      END IF
C
C     FORMAT STATEMENTS
C     =================
100   FORMAT(A70)
210   FORMAT(5X,'MPNE ANALYTICAL SOLUTION FOR 1-D TRANSPORT',/,
     1       5X,'VERSION 4.1',/,
     2       5X,'===========',/)
211   FORMAT(5X,A60,/)
220   FORMAT(5X,'BASIC INPUT DATA',/,
     1       5X,'----------------',/,
     1       5X,'RHOb  : BULK DENSITY                       ',1PE12.6/
     1       5X,'THETA : TOTAL WATER CONTENT                ',1PE12.6/
     1       5X,'Q     : DARCY FLUX                         ',1PE12.6/
     1       5X,'D     : HYDRODYNAMIC DISPERSION COEFF.     ',1PE12.6)
221   FORMAT(/5X,'MODEL SPECIFIC DATA',/,
     1       5X,'-------------------',/,
     1       5X,'PHI   : PROPORTION OF MOBILE PORE WATER    ',1PE12.6/
     1       5X,'f     : MASS FRACTION OF SORBENT COMPRISING',/,
     1       5X,'        MOBILE REGION                      ',1PE12.6/
     1       5X,'ALFA  : FIRST-ORDER MASS TRANSFER COEFF.   ',1PE12.6/
     1       5X,'FM    : FRACTION OF SORBENT IN MOBILE REGION',/,
     1       5X,'        FOR INSTANTANEOUS SORPTION         ',1PE12.6/
     1       5X,'FIM   : FRACTION OF SORBENT IN IMMOBILE REGION',/,
     1       5X,'        FOR INSTANTANEOUS SORPTION         ',1PE12.6/
     1       5X,'KM    : MOBILE EQUIL. SORPTION COEFF.      ',1PE12.6/
     1       5X,'KIM   : IMMMOBILE EQUIL. SORPTION COEFF.   ',1PE12.6/
     1       5X,'km2   : FIRST-ORDER SORPTION KINETIC COEFF.',/,
     1       5X,'        FOR MOBILE REGION                  ',1PE12.6/
     1       5X,'kim2  : FIRST-ORDER SORPTION KINETIC COEFF.',/,
     1       5X,'        FOR IMMOBILE REGION                ',1PE12.6)
222   FORMAT(/5X,'FIRST-ORDER DECAY COEFFICIENTS',/,
     1       5X,'------------------------------',/,
     1       5X,'LM     : MOBILE REGION                     ',1PE12.6/
     1       5X,'LM1    : MOBILE REGION INSTANTANEOUS SITES ',1PE12.6/
     1       5X,'LM2    : MOBILE REGION RATE-LIMITED SITES  ',1PE12.6/
     1       5X,'LIM    : IMMOBILE REGION                   ',1PE12.6/
     1       5X,'LIM1   : IMMOBILE REGION FOR INSTANTANEOUS ',/,
     1       5X,'         SORPTION SITES                    ',1PE12.6/
     1       5X,'LIM2   : IMMOBILE REGION FOR RATE-LIMITED  ',/,
     1       5X,'         SORPTION SITES                    ',1PE12.6)
231   FORMAT(/5X,'INITIAL CONCENTRATIONS',/,
     1       5X,'----------------------',/,
     1       5X,'CM0   :                                    ',1PE12.6/
     1       5X,'CIM0  :                                    ',1PE12.6/
     1       5X,'SM20  :                                    ',1PE12.6/
     1       5X,'SIM20 :                                    ',1PE12.6)
240   FORMAT(/5X,'CALCULATED DIMENSIONLESS PARAMETERS',/,
     1       5X,'-----------------------------------',/,
     1       5X,'R     : TOTAL RETARDATION FACTOR           ',1PE12.6/
     1       5X,'P     : PECLET NUMBER                      ',1PE12.6/
     1       5X,'OMEGA : DAMKOHLER # REPRESENTING PNE       ',1PE12.6/
     1       5X,'KM0   : DAMKOHLER # REPRESENTING MOBILE',/,
     1       5X,'        REGION SORPTION NON-EQUILIBRIUM    ',1PE12.6/
     1       5X,'KIM0  : DAMKOHLER # REPRESENTING IMMOBILE',/,
     1       5X,'        REGION SORPTION NON-EQUILIBRIUM    ',1PE12.6)
241   FORMAT(/,5X,'CALCULATED FRACTIONAL RETARDATION FACTORS',/,
     1       5X,'-----------------------------------------',/,
     1       5X,'BETA1                                       ',1PE12.6/
     1       5X,'BETA2                                       ',1PE12.6/
     1       5X,'BETA3                                       ',1PE12.6/
     1       5X,'BETA4                                       ',1PE12.6)
242   FORMAT(/,5X,'CALCULATED DIMENSIONLESS DECAY PARAMETERS',/,
     1       5X,'-----------------------------------------',/,
     1       5X,'EM1                                         ',1PE12.6/
     1       5X,'EM2                                         ',1PE12.6/
     1       5X,'EIM1                                        ',1PE12.6/
     1       5X,'EIM2                                        ',1PE12.6)
230   FORMAT(/5X,'INFLOW BOUNDARY CONDITION DATA',/,
     1       5X,'------------------------------',/,
     1       5X,'INFLOW BOUNDARY CONDITION TYPE  ',I2)
1234  FORMAT(//5X,'CONSTRUCTED INFLOW CONCENTRATION HISTOGRAM',/,
     1       15X,'TIME INTERVAL',11X,'CONCENTRATION',/,5X,48('-'))
1235  FORMAT(5X,1PE12.6,' - ',1PE12.6,6X,1PE12.6)
1236  FORMAT(5X,1PE12.6,' --> INFINITY ',7X,1PE12.6)
2231  FORMAT(/5X,'OUTFLOW BOUNDARY CONDITION DATA',/,
     1       5X,'-------------------------------',/,
     1       5X,'OUTFLOW BOUNDARY CONDITION TYPE ',I2/
     1       5X,'LENGTH OF DOMAIN (FOR FINITE DOMAIN)        ',1PE12.6)
2232  FORMAT(5X,'CONCENTRATION AT OUTFLOW BOUNDARY, CL       ',1PE12.6)
500   FORMAT(//10X,'TIME',13X,'X',17X,'Cm',18X,'Cim'/,5X,70('-'))
510   FORMAT(5X,1PE12.6,5X,1PE12.6,5X,1PE15.6,5X,1PE15.6)
C
C     TERMINATE MAIN PROGRAM
C     ======================
      CLOSE(55)
      CLOSE(66)
      CLOSE(67)
      END
C
C     ***************************
      COMPLEX*16 FUNCTION FBAR(P)
C     ***************************
c     Laplace-transformed solution
c
c     DECLARATION OF VARIABLES
c     ========================
      IMPLICIT NONE
      INTEGER MAXPT
      PARAMETER(MAXPT=50)
      DOUBLE PRECISION PARAM(19)
      DOUBLE PRECISION CM0,CIM0,SM20,SIM20,DELTA,LENGTH,CL,XX
      DOUBLE PRECISION TIME,TS(MAXPT),DELC(MAXPT)
      DOUBLE PRECISION RHOB,THETA,Q,D,PHI,THM,THIM,F,ALFA,FM,FIM,
     1                 KM,KIM,KM2,KIM2,LM,LM1,LM2,LIM,LIM1,LIM2
      DOUBLE PRECISION PCHK,FCHK
      COMPLEX*16 P,SUM,TERM,FS,G10,G20,GAM1,GAM2,GAM3,GAM4,B,H1,H2,
     1           D1,D2,D3,E1,E2
      COMPLEX*16 D4,D5,D6,CDEXP
      COMPLEX*16 CMB,CIMB
      INTEGER OBC,NP,N,IC
C
      COMMON /ICS/CM0,CIM0,SM20,SIM20
      COMMON /BCS/TIME,DELTA,TS,DELC,NP,OBC,LENGTH,CL,XX
      COMMON /MPNE/PARAM
      COMMON /CSWITCH/IC
C
C     "LOAD" MPNE PARAMETERS FROM THE PARAM ARRAY
C     ===========================================
      RHOB  = PARAM( 1)
      THETA = PARAM( 2)
      Q     = PARAM( 3)
      D     = PARAM( 4)
      PHI   = PARAM( 5)
      F     = PARAM( 6)
      ALFA  = PARAM( 7)
      FM    = PARAM( 8)
      FIM   = PARAM( 9)
      KM    = PARAM(10)
      KIM   = PARAM(11)
      KM2   = PARAM(12)
      KIM2  = PARAM(13)
      LM    = PARAM(14)
      LM1   = PARAM(15)
      LM2   = PARAM(16)
      LIM   = PARAM(17)
      LIM1  = PARAM(18)
      LIM2  = PARAM(19)
C
C     CALCULATE POROSITIES
C     ====================
      THM  = PHI*THETA
      THIM = (1.D0-PHI)*THETA
C
C     TRAP POTENTIAL DIVISION BY ZERO FOR CASE OF NO IMMOBILE REGION
C     ==============================================================
      PCHK = DABS(PHI-1.D0)
      FCHK = DABS(F-1.D0)
C
      IF((PCHK.LT.1.D-10).AND.(FCHK.LT.1.D-10)) THEN
         ALFA=1.D0
      ENDIF
C
c     CALCULATE TRANSFORMED INFLOW CONCENTRATION HISTORY
c     ==================================================
      sum = CMPLX(0.d0,0.d0,KIND=8)
      do n=1,np
         if(ts(n).lt.time) then
            if(ts(n).le.0.d0) then
               term = delc(n)/p
            else
               term = (delc(n)/p)*(cdexp(-p*ts(n)))
            end if
         else
            term = CMPLX(0.d0,0.d0,KIND=8)
         end if
         sum = sum+term
      end do
      fs = sum
C
C     CALCULATE INITIAL CONDITION TERMS
C     =================================
      G10 = (((1.D0-F)*RHOB*KIM2/(P+KIM2+LIM2))*SIM20
     1      +(THIM+(1.D0-F)*RHOB*FIM*KIM)*CIM0)/
     2      ((P*(THIM+(1.D0-F)*RHOB*FIM*KIM))+(THIM*LIM)
     3      +((1.D0-F)*RHOB*LIM1*FIM*KIM)
     4      +((1.D0-F)*RHOB*(1.D0-FIM)*KIM*KIM2*(P+LIM2)/
     5       (P+KIM2+LIM2) + ALFA))
      G20 = (THM+F*RHOB*FM*KM)*CM0
     1      +((F*RHOB*KM2)/(P+KM2+LM2))*SM20
C
C     CALCULATE GAMMA TERMS
C     =====================
      GAM1 = (1.D0+(F*RHOB*FM*KM)/THM)*P
      GAM2 = (F*RHOB*(1.D0-FM)*KM*KM2*((P+LM2)/
     1       (P+KM2+LM2)))/THM
      GAM3 = (ALFA-((ALFA*ALFA)/
     1       ((THIM+(1.D0-F)*RHOB*FIM*KIM)*P
     2       +(THIM*LIM)+((1.D0-F)*RHOB*LIM1*FIM*KIM)
     3       +((1.D0-F)*RHOB*(1.D0-FIM)*KIM*KIM2
     4       *((P+LIM2)/(P+KIM2+LIM2)))+ALFA)))/THM
      GAM4 = LM+(F*RHOB*LM1*FM*KM)/THM
C
C     CALCULATE COEFFICIENTS
C     ======================
      B  = THM*(GAM1+GAM2+GAM3+GAM4)
      H1 = (Q-((Q*Q)+4.D0*B*THM*D)**0.5)/(2.D0*THM*D)
      H2 = (Q+((Q*Q)+4.D0*B*THM*D)**0.5)/(2.D0*THM*D)
C
C     CALCULATE SOLUTION FOR SEMI-FINITE DOMAIN
C     =========================================
      IF(OBC.EQ.1) THEN
C
         IF((DABS(Q).LE.0.D0).AND.(DELTA.LE.0.D0)) THEN
		  E1 = FS-(ALFA*G10+G20)/B
	   ELSE
            E1 = (Q/(Q-THM*DELTA*D*H1))*(FS-(ALFA*G10+G20)/B)
         END IF
         CMB = E1*CDEXP(H1*XX) + ((ALFA*G10)+G20)/B
C
      ELSE
C
C     CALCULATE SOLUTION FOR FINITE DOMAIN, TYPE II
C     =============================================
      IF(OBC.EQ.2) THEN
C
         IF((DABS(Q).LE.0.D0).AND.(DELTA.LE.0.D0)) THEN
            D1 = H2*CDEXP(H2*LENGTH)-H1*CDEXP(H1*LENGTH)
            D2 = (FS-((ALFA*G10)+G20)/B)*H2*CDEXP(H2*LENGTH)
            D3 =-(FS-((ALFA*G10)+G20)/B)*H1*CDEXP(H1*LENGTH)
	   ELSE
            D1 = H2*(Q-THM*DELTA*D*H1)*CDEXP(H2*LENGTH)
     1          -H1*(Q-THM*DELTA*D*H2)*CDEXP(H1*LENGTH)
            D2 = q*(FS-((ALFA*G10)+G20)/B)*H2*CDEXP(H2*LENGTH)
            D3 =-q*(FS-((ALFA*G10)+G20)/B)*H1*CDEXP(H1*LENGTH)
         END IF
         E1  = D2/D1
         E2  = D3/D1
         CMB = E1*CDEXP(H1*XX) + E2*CDEXP(H2*XX)
     1        + ((ALFA*G10)+G20)/B
C
      ELSE
C
C     CALCULATE SOLUTION FOR FINITE DOMAIN, TYPE I
C     ============================================
      IF(OBC.EQ.3) THEN
C
         IF((DABS(Q).LE.0.D0).AND.(DELTA.LE.0.D0)) THEN
            D4 = CDEXP(H2*LENGTH)-CDEXP(H1*LENGTH)
            D5 = (FS-((ALFA*G10)+G20)/B)*CDEXP(H2*LENGTH)
     1          -(CL/P-((ALFA*G10)+G20)/B)
            D6 =-(FS-((ALFA*G10)+G20)/B)*CDEXP(H1*LENGTH)
     1          +(CL/P-((ALFA*G10)+G20)/B)
	   ELSE
            D4 = (Q-THM*DELTA*D*H1)*CDEXP(H2*LENGTH)
     1          -(Q-THM*DELTA*D*H2)*CDEXP(H1*LENGTH)
            D5 = q*(FS-((ALFA*G10)+G20)/B)*CDEXP(H2*LENGTH)
     1          -(CL/P-((ALFA*G10)+G20)/B)*(Q-THM*DELTA*D*H2)
            D6 =-q*(FS-((ALFA*G10)+G20)/B)*CDEXP(H1*LENGTH)
     1          +(CL/P-((ALFA*G10)+G20)/B)*(Q-THM*DELTA*D*H1)
         END IF
         E1  = D5/D4
         E2  = D6/D4
         CMB = E1*CDEXP(H1*XX) + E2*CDEXP(H2*XX)
     1        + ((ALFA*G10)+G20)/B
C
      END IF
      END IF
      END IF
C
C     INVERT LAPLACE-TRANSFORM SOLUTIONS FOR EITHER Cm OR Cim
C     =======================================================
      IF(IC.EQ.1) THEN
         FBAR = CMB
      ELSE
         IF((PCHK.LT.1.D-10).AND.(FCHK.LT.1.D-10)) THEN
            CIMB = CMPLX(0.d0,0.d0,KIND=8)
         ELSE
            CIMB = CMB*(alfa/
     1            (p*(thim+(1.d0-f)*rhob*Fim*Kim)+thim*lim
     2           +(1.d0-f)*rhob*lim1*Fim*Kim
     3           +(1.d0-f)*rhob*(1.d0-Fim)*Kim*kim2
     4           *((p+lim2)/(p+kim2+lim2))
     5           +alfa)) + G10
         END IF
         FBAR = CIMB
      END IF
C
      RETURN
      END FUNCTION FBAR
C
C     ******************
      FUNCTION CDEXP(Z2)
C     ******************
C     COMPLEX DOUBLE PRECISION EXPONENTIAL FUNCTION
C     Coded by JP Keizer, 2002 01
c
      COMPLEX (KIND=8) :: CDEXP
      COMPLEX (KIND=8), INTENT(IN) :: Z2
      REAL (KIND=8) :: A                    ! real part of z
      REAL (KIND=8) :: B                    ! imaginary part of z
c
      A     = REAL(Z2,KIND=8)
      B     = AIMAG(Z2)
      CDEXP = DEXP(A)*(CMPLX(DCOS(B),DSIN(B),KIND=8))
c
      END FUNCTION CDEXP
c
c     *******************
      INCLUDE 'HOOGD.FOR'
